import os
import pandas as pd
from datetime import datetime
from creon_api.config import CODE_LIST_FOLDER_PATH, ADJ_INFO_PATH


def load_krx_code_list(date: str = None):
    if date is None:
        date = datetime.today().strftime("%Y%m%d")

    krx_code_list = []
    for market_folder in ['거래소', '코스닥']:
        try:
            code_df = pd.read_csv(f"{CODE_LIST_FOLDER_PATH}/{market_folder}/{date}.csv", dtype="object")
        except FileNotFoundError as e:
            raise FileNotFoundError(f"종목리스트 파일이 없습니다. {e}")
        krx_code_list += code_df['code'].values.tolist()
    return krx_code_list


def get_adj_file_code_list():
    file_list = os.listdir(f"{ADJ_INFO_PATH}")
    code_list = [file.split(".")[0] for file in file_list]
    return code_list


index_code_list = [
    "U001",  # 종합주가지수
    "U002",  # 대형(시가총액)
    "U003",  # 중형(시가총액)
    "U004",  # 소형(시가총액)
    "U005",  # 음식료품
    "U006",  # 섬유, 의복
    "U007",  # 종이, 목재
    "U008",  # 화학
    "U009",  # 의약품
    "U010",  # 비금속광물
    "U011",  # 철강, 금속
    "U012",  # 기계
    "U013",  # 전기, 전자
    "U014",  # 의료정밀
    "U015",  # 운송장비
    "U016",  # 유통업
    "U017",  # 전기가스업
    "U018",  # 건설업
    "U019",  # 운수창고
    "U020",  # 통신업
    "U021",  # 금융업
    "U022",  # 은행
    "U024",  # 증권
    "U025",  # 보험
    "U026",  # 서비스업
    "U027",  # 제조업
    "U130",  # KOSPI200 경기방어소비재
    "U131",  # KOSPI200 중소형주지수
    "U132",  # KOSPI200 산업재
    "U133",  # KOSPI200 헬스케어
    "U141",  # KOSPI200 건설
    "U142",  # KOSPI200 중공업
    "U143",  # KOSPI200 철강 / 소재
    "U144",  # KOSPI200 에너지 / 화학
    "U145",  # KOSPI200 정보기술
    "U146",  # KOSPI200 금융
    "U147",  # KOSPI200 생활소비재
    "U148",  # KOSPI200 경기소비재
    "U149",  # 코스피 200 에너지화학 레버리지지수
    "U150",  # 미국달러선물 레버리지지수
    "U151",  # 미국달러선물지수
    "U152",  # 미국달러선물인버스지수
    "U153",  # 코스피 200 커버드콜 5 % OTM
    "U154",  # 코스피 200 프로텍티브풋 OTM 5 %
    "U155",  # KOSPI200 동일가중지수
    "U156",  # KOSPI100 동일가중지수
    "U157",  # KOSPI50 동일가중지수
    "U165",  # 코스피 200 정보기술 레버리지지수
    "U166",  # 코스피 200 금융 레버리지지수
    "U167",  # 코스피 200 경기소비재 레버리지지수
    "U170",  # KTOP 30
    "U177",  # KOSPI200레버리지지수
    "U178",  # F - KOSPI200인버스지수
    "U179",  # KOSPI200선물지수
    "U180",  # KOSPI200지수
    "U181",  # KOSPI100지수
    "U182",  # KOSPI50지수
    "U190",  # 코스피 200 고배당지수
    "U191",  # 코스피 200 저변동성지수
    "U192",  # 코스피 고배당 50지수
    "U193",  # 코스피 배당성장 50지수
    "U194",  # 코스피 우선주 지수
    "U195",  # 코스피 200 가치저변동성 지수
    "U196",  # 코스피 200 건설 레버리지지수
    "U197",  # 코스피 200 중공업 레버리지지수
    "U198",  # 코스피 200 헬스케어 레버리지지수
    "U501",  # 국채선물지수
    "U502",  # 국채선물인버스지수
    "U503",  # 국채선물10년물지수
    "U504",  # 국채선물10년물인버스지수
    "U505",  # 국채선물대표(3 - 10)지수
    "U506",  # 국채선물대표(3 - 10) 레버리지지수
    "U507",  # 국채선물대표(3 - 10) 인버스지수
    "U508",  # 국채선물대표(3 - 10) 인버스 - 2X지수
    "U511",  # 주식골드지수
    "U512",  # 주식미국채DAE지수
    "U513",  # 주식국채혼합(주식형)지수
    "U514",  # 주식국채혼합(채권형)지수
    "U530",  # MSCI Korea Index
    "U541",  # 코스피200리스크컨트롤6 % 지수
    "U542",  # 코스피200리스크컨트롤8 % 지수
    "U543",  # 코스피200리스크컨트롤10 % 지수
    "U544",  # 코스피200리스크컨트롤12 % 지수
    "U545",  # KOSPI200 변동성지수
    "U546",  # 코스피 200 DAE지수
    # "U547",  # 코스피 200 선물 플러스지수 -> 유효하지않음
    "U548",  # K200 USD 선물 바이셀지수
    "U549",  # USD K200 선물 바이셀지수
    "U550",  # 코스피 선물매수 콜매도 지수
    "U551",  # 코스피 선물매도 풋매도 지수
    "U552",  # 코스피 200 선물 인버스 - 2X지수
    "U553",  # 코스피 200 선물 인버스 - 3X지수
    "U554",  # 미국달러선물 인버스 - 2X지수
    "U555",  # 미국달러선물 인버스 - 3X지수
    "U560",  # 엔선물지수
    "U561",  # 엔선물 레버리지 지수
    "U562",  # 엔선물 인버스지수
    "U563",  # 엔선물 인버스 - 2X지수
    "U564",  # 엔선물 인버스 - 3X지수
    "U565",  # 유로선물지수
    "U566",  # 유로선물 레버리지 지수
    "U567",  # 유로선물 인버스지수
    "U568",  # 유로선물 인버스 - 2X지수
    "U569",  # 유로선물 인버스 - 3X지수
    "U570",  # 코스피 200 선물 레버리지 지수
    "U576",  # 미니 코스피 200 선물지수
    "U577",  # 미니 코스피 200 선물 레버리지지수
    "U578",  # 미니 코스피 200 선물 인버스지수
    "U579",  # 미니 코스피 200 선물 인버스 - 2X지수
    "U583",  # 코스피 200 밸류 가중 지수
    "U584",  # 코스피 200 퀄리티 가중 지수
    "U585",  # 코스피 200 모멘텀 가중 지수
    "U586",  # 코스피 200 로우볼 가중 지수
    "U587",  # 코스피 200 커버드콜 ATM
    "U588",  # 코스피 200 현선물레버리지 1.5X 지수
    "U201",  # (코스닥)종합
    "U202",  # (코스닥)KOSDAQ 100
    "U203",  # (코스닥)KOSDAQ MID 300
    "U204",  # (코스닥)KOSDAQ SMALL
    "U212",  # (코스닥)기타
    "U215",  # (코스닥)IT종합
    "U224",  # (코스닥)제조
    "U226",  # (코스닥)건설
    "U227",  # (코스닥)유통
    "U229",  # (코스닥)운송
    "U231",  # (코스닥)금융
    "U237",  # (코스닥)오락, 문화
    "U241",  # (코스닥)통신방송서비스
    "U242",  # (코스닥)IT S / W & SVC
    "U243",  # (코스닥)IT H / W
    "U256",  # (코스닥)음식료, 담배
    "U258",  # (코스닥)섬유, 의류
    "U262",  # (코스닥)종이, 목재
    "U263",  # (코스닥)출판, 매체복제
    "U265",  # (코스닥)화학
    "U266",  # (코스닥)제약
    "U267",  # (코스닥)비금속
    "U268",  # (코스닥)금속
    "U270",  # (코스닥)기계, 장비
    "U272",  # (코스닥)일반전기전자
    "U274",  # (코스닥)의료, 정밀기기
    "U275",  # (코스닥)운송장비, 부품
    "U277",  # (코스닥)기타 제조
    "U310",  # 코스닥 150 기술주
    "U311",  # 코스닥 150 소재
    "U312",  # 코스닥 150 산업재
    "U313",  # 코스닥 150 필수소비재
    "U314",  # 코스닥 150 자유소비재
    "U315",  # 코스닥 150 정보기술
    "U316",  # 코스닥 150 생명기술
    "U317",  # 코스닥 150 문화기술
    "U351",  # (코스닥)통신서비스
    "U352",  # (코스닥)방송서비스
    "U353",  # (코스닥)인터넷
    "U354",  # (코스닥)디지털컨텐츠
    "U355",  # (코스닥)소프트웨어
    "U356",  # (코스닥)컴퓨터서비스
    "U357",  # (코스닥)통신장비
    "U358",  # (코스닥)정보기기
    "U359",  # (코스닥)반도체
    "U360",  # (코스닥)IT부품
    "U381",  # (코스닥)우량기업
    "U382",  # (코스닥)벤처기업
    "U383",  # (코스닥)중견기업
    "U384",  # (코스닥)기술성장기업
    "U390",  # 코스닥 150
    "U391",  # 코스닥 150 동일가중지수
    "U392",  # 코스닥 150 레버리지지수
    "U393",  # 코스닥 150 선물지수
    "U394",  # 코스닥 150 선물 인버스지수
    "U395",  # 코스닥 150 선물 인버스 - 2X지수
    "U396",  # 코스닥 150 선물 인버스 - 3X지수
    "U397",  # 코스닥 150 레버리지 0.5지수
    "U398",  # 코스닥 150 저변동성
    "U399",  # 코스닥 150선물 레버리지지수
    "U100",  # KRX100지수
    "U111",  # KRX 자동차
    "U112",  # KRX 반도체
    "U113",  # KRX 헬스케어
    "U114",  # KRX 은행
    "U116",  # KRX 에너지화학
    "U117",  # KRX 철강
    "U119",  # KRX 방송통신
    "U120",  # KRX 건설
    "U122",  # KRX 증권
    "U123",  # KRX 기계장비
    "U124",  # KRX 보험
    "U125",  # KRX 운송
    "U128",  # KRX ESG Leaders 150 지수
    "U129",  # KRX Governance Leaders 100 지수
    "U134",  # KRX Eco Leaders 100 지수
    "U158",  # KRX100 동일가중지수
    "U164",  # KRX 고배당 50지수
    "U171",  # KTOP 30 레버리지 지수
    "U515",  # 경기방어소비재 채권혼합지수
    "U516",  # 배당성장채권혼합지수
    "U556",  # KRX 스마트 퀄리티 지수
    "U557",  # KRX 스마트 밸류 지수
    "U558",  # KRX 스마트 모멘텀 지수
    "U559",  # KRX 밸류모멘텀 지수
]


def make_dir(path: str):
    temp_path = ""
    for path in path.split('/'):
        temp_path += (path + "/")
        if not is_exist(temp_path):
            os.mkdir(temp_path)


def is_exist(path: str) -> bool:
    return os.path.exists(path)
